CLIENT_MAX_PAGES = 100
AMOUNT_DESCRIPTION = (
    "The amount is an integer in smallest denomination to avoid precision loss.\n"
    "- So 1000 refers to 1000 cents or $10.00\n"
)
SK_CATEGORIES = {
    1: "Pet",
    2: "Other",
    3: "Office",
    4: "Airlines",
    5: "Car rental",
    6: "Lodging",
    7: "Travel misc",
    8: "Taxi and rideshare",
    9: "Freight, moving and delivery services",
    10: "Shipping",
    11: "Utilities",
    12: "Office supplies and cleaning",
    13: "General merchandise",
    14: "Electronics",
    15: "Clothing",
    16: "Books and newspapers",
    17: "Supermarkets and grocery stores",
    18: "Fuel and gas",
    19: "Restaurants",
    20: "Alcohol and bars",
    21: "Medical",
    23: "Fees and financial institutions",
    24: "Entertainment",
    25: "Professional services",
    26: "Taxes and tax preparation",
    27: "Advertising",
    28: "Parking",
    29: "Car services",
    30: "Gambling",
    31: "Clubs and memberships",
    32: "Legal",
    33: "Education",
    34: "Charitable donations",
    35: "Political organizations",
    36: "Religious organizations",
    37: "Fines",
    38: "Government services",
    39: "Intra-company purchases",
    40: "SaaS / Software",
    41: "Cloud computing",
    42: "Streaming services",
    43: "Internet and phone",
    44: "Insurance",
}
